.586
.model flat, stdcall
option casemap:none

; Service Skeleton
;	An example of a service application. It beeps every second :)
;
; This is based on:
;	* "How to Write NT Services in Assembly Language" by Cynical Pinnacle
;   	- http://win32assembly.online.fr/tutorials.html"
;	* Microsoft MSDN Service reference Svc.cpp
;		- http://msdn2.microsoft.com/en-us/library/bb540475(VS.85).aspx
;	* A lot of googling for examples and ideas
;	* The MASM forum
;		- http://www.masm32.com/board/
;
;	I hope you find it useful - Don Reid 2008.
;
;	Search for:
;   	*TODO 	- things that I recommend be changed for your service
;		TODO	- things that are optional

;	  Command line options
;		-install
;			install the service
;		-uninstall
;			remove the service
;		anything else displays the Version
; 
; Written using (and a big thanks to everyone involved in):
;		masm32		http://www.masm32.com/
;		WinAsm		http://www.winasm.net/
;
;
; Version
;	0.0801.23
;		- Released to http://www.masm32.com/board/ for general consumption
 

include \masm32\include\windows.inc
include \masm32\include\kernel32.inc
include \masm32\include\user32.inc
include \masm32\include\shell32.inc
include \masm32\include\masm32.inc
include \masm32\include\advapi32.inc

includelib \masm32\lib\kernel32.lib 
includelib \masm32\lib\user32.lib
includelib \masm32\lib\shell32.lib
includelib \masm32\lib\masm32.lib
includelib \masm32\lib\advapi32.lib

; Define the structure required to add a description for the service
SERVICE_DESCRIPTION	STRUCT
	lpDescription	dd	?
SERVICE_DESCRIPTION EndS

InstallService	PROTO stdcall
RemoveService	PROTO stdcall
SvcMain			PROTO stdcall	:DWORD,	:DWORD
SvcCtrlHandler	PROTO stdcall	:DWORD
ReportSvcStatus	PROTO stdcall	:DWORD, :DWORD, :DWORD, :DWORD
SvcInit			PROTO stdcall	:DWORD,	:DWORD
SvcTerminate	PROTO stdcall	:DWORD
SvcThread		PROTO stdcall	:DWORD
SvcLogError		PROTO stdcall	:DWORD, :DWORD,	:DWORD
PutInEventLog	PROTO stdcall	:DWORD

;_______________________________________________________________________________
.const
cVersion 		equ " 0.0801.23"

; TODO - Add "or SERVICE_INTERACTIVE_PROCESS" to cSvcStartType
cSvcStartType	equ	SERVICE_WIN32_OWN_PROCESS	; if you want an interactive process

; TODO - Add "or SERVICE_ACCEPT_PAUSE_CONTINUE" to cSvcControls
cSvcControls	equ	SERVICE_ACCEPT_SHUTDOWN \	; if you want to have pause and resume functions.
					or SERVICE_ACCEPT_STOP		; See SvcCtrlHandler for simple Pause/Resume code

pszTemp			equ	ADDR szTemp					; Pointer to the romper room
SERVICE_CONFIG_DESCRIPTION	equ 1				; Not defined in Windows.inc

;_______________________________________________________________________________
.data?
hInstance		HINSTANCE	?
CommandLine		LPCWSTR		?

szTemp			db	10240 dup(?)				; 10k romper room
szPath			db	MAX_PATH dup(?)				; Apps full file name
bServicePaused	dd	?							; Service Paused state

;_______________________________________________________________________________
.data
; *TODO - Change szServiceName to the name of your service
szServiceName	db	"ServSkeleton",0
; *TODO - Change szDisplayName to the friendly name of your service
szDisplayName	db	"Service Skeleton",0
; *TODO - Change szDescription to the Description for your service
szDescription	db	"Skeleton service for you to extend as you like.",0

szVersionString	db	" Version ", cVersion,13,10,10,
					"   Usage:",13,10,
					"     -install      Installs the service",13,10,
					"     -uninstall    Removes the service",13,10,10,0
					

; Boolean command line switches and their respective switch
szInstall		db	"-install",0
szUninstall		db	"-uninstall",0

; Messages
szGetModFN		db	"GetModuleFileName",0
szOpenSCM		db	"OpenSCManager",0
szCreateSvc		db	"CreateService",0
szOpenSvc		db	"OpenService",0
szDelSvc		db	"DeleteService",0
szSCtrlDisp		db	"StartServiceCtrlDispatcher",0
szRegCtrlHand	db	"RegisterServiceCtrlHandler",0
szCreateEvent	db	"CreateEvent",0
szCreateThread	db	"CreateThread", 0
szEventLog		db	"%s failed (Error %d)",0

; Service stuff
szInstalledOK	db	"Service installed successfully.",13,10,0
szUnInstalledOK	db	"Service uninstalled successfully.",13,10,0
pSvcConfDesc	SERVICE_DESCRIPTION <>

SvcStatus		SERVICE_STATUS	<>
hSvcStatus		SERVICE_STATUS_HANDLE	NULL
hSvcStopEvent	HANDLE		NULL
schSCManager	SC_HANDLE	NULL
schService		SC_HANDLE	NULL
hThread			HANDLE		NULL

; Service table entries - array must be null terminated
SvcTable		SERVICE_TABLE_ENTRY	<0,0>
; TODO - 	Add more service entries here.
;			Do not change SvcTableEnd as it terminates the array
SvcTableEnd		SERVICE_TABLE_ENTRY	<0,0>

;_______________________________________________________________________________
.code
start:
	; Get and save the instance handle
	invoke	GetModuleHandle,NULL
	mov		hInstance, eax
	
	; Get and save the pointer to the command line
	invoke	GetCommandLine
	mov		CommandLine, eax
	
	; Copy the first command line paramter to the romper room
	invoke	GetCL, 1, ADDR szTemp

	; Was there a parameter?
	.if eax == 1
		; Yes - convert it to lower case
		invoke	szLower, ADDR szTemp
		
		; check for "-uninstall" ignoring switch character (0 if no match)
		invoke	szCmp, pszTemp, ADDR szUninstall
		.if eax != 0
			; Uninstall the service
			invoke RemoveService
		.else
			; check for "install" ignoring switch character (0 if no match)
			invoke	szCmp, pszTemp, ADDR szInstall
			.if eax != 0
				; Install the service
				invoke	InstallService
			.else
				; Some cruft on the command line - just display the version information
				invoke	StdOut, ADDR szVersionString	
			.endif
		.endif
		
	.else
		; No command line parameter - Assume that this is a service start
		;   we pick it up later in InstallService if this is not the case
		lea		eax, szServiceName
		mov		SvcTable.lpServiceName, eax
		mov		SvcTable.lpServiceProc, offset SvcMain
		
		; Returns when service thread has stopped
		invoke	StartServiceCtrlDispatcher, ADDR SvcTable

		.if eax == 0
			; Service Failed - get the error.
			invoke	GetLastError
		
			; Check if this program was just run from the command line
			.if eax == ERROR_FAILED_SERVICE_CONTROLLER_CONNECT
				; Yes - display the version information
				invoke	StdOut, ADDR szVersionString	
			.else
				; No - something went wrong - put the error in the event log
				invoke	GetLastError
				invoke	SvcLogError, ADDR szSCtrlDisp, eax, TRUE
			.endif
		.endif
	.endif
	
	; All done - time to quit
	Invoke ExitProcess,NULL

;_______________________________________________________________________________
InstallService proc
	; Installs the service
	
	; Get the full app name - needed in CreateService
	invoke	GetModuleFileName, hInstance, ADDR szPath, MAX_PATH

	.if eax == 0
		; GetModule failed - report error and quit
		invoke	GetLastError
		invoke	SvcLogError, ADDR szGetModFN, eax, FALSE
	.else
		; Get handle to the SCM database
		invoke	OpenSCManager,NULL,NULL,GENERIC_WRITE
		mov		schSCManager, eax
		
		.if eax == 0
			; Open failed - report error and quit
			invoke	GetLastError
			invoke	SvcLogError, ADDR szOpenSCM, eax, FALSE
		.else		
			; Create the service and save the handle
			invoke	CreateService, 	schSCManager, ADDR szServiceName, ADDR szDisplayName,
									SERVICE_ALL_ACCESS, cSvcStartType, SERVICE_DEMAND_START,
									SERVICE_ERROR_NORMAL, ADDR szPath, NULL, NULL, NULL, NULL, NULL
			mov		schService, eax
		
			; Did it install?
			.if eax == 0
				; Install failed - report error and quit
				invoke	GetLastError
				invoke	SvcLogError, ADDR szCreateSvc, eax, FALSE
			
				; Close the service handle
				invoke CloseServiceHandle, schSCManager
			.else
				; Installed OK - add a Description to the service
				lea	eax, szDescription
				mov	pSvcConfDesc.lpDescription, eax
				invoke	ChangeServiceConfig2,schService , SERVICE_CONFIG_DESCRIPTION, ADDR pSvcConfDesc

				; Tell the world
				invoke	StdOut, ADDR szInstalledOK
			
				; Close the handles
				invoke	CloseServiceHandle,schService
				invoke	CloseServiceHandle,schSCManager
			.endif
		.endif
	.endif
	
	Ret
	
InstallService EndP

;_______________________________________________________________________________
RemoveService proc
	; Removes the Service
	
	; Get handle to the SCM database
	invoke	OpenSCManager,NULL,NULL,GENERIC_WRITE
	mov		schSCManager, eax

	.if	eax == 0
		; Open failed - report error and quit
		invoke	GetLastError
		invoke	SvcLogError, ADDR szOpenSCM, eax, FALSE
	.else
		; Open the service
		invoke	OpenService, schSCManager, ADDR szServiceName, DELETE
		mov		schService, eax
	
		.if eax == 0
			; OpenService failed - report error and quit
			invoke	GetLastError
			invoke	SvcLogError, ADDR szOpenSvc, eax, FALSE
		
			; Close the service handle
			invoke CloseServiceHandle, schSCManager
		.else				
			; Delete the service
			invoke	DeleteService, schService
		
			.if eax != 0
				; Delete succeeded
				invoke	StdOut, ADDR szUnInstalledOK
			.else
				; Delete Failed
				invoke	SvcLogError, ADDR szDelSvc, eax, FALSE
			.endif
			
			; Close the handles
			invoke	CloseServiceHandle,schService
			invoke	CloseServiceHandle,schSCManager
		.endif
	.endif

	Ret
	
RemoveService EndP

;_______________________________________________________________________________
SvcMain proc dwArgc: DWORD, lpszArgv: DWORD
	; Main proc for the service - this is where we get the worker thread ready, start it
	;	and then wait for it to be stopped.
	 
	; Register the handler proc for the service
	invoke	RegisterServiceCtrlHandler, ADDR szServiceName, SvcCtrlHandler
	mov		hSvcStatus, eax
	
	; Did the registration succeed?
	.if	eax == 0
		; No - Save an error and then quit
		invoke	GetLastError
		invoke	SvcLogError, ADDR szRegCtrlHand, eax, TRUE

		; Terminate the service
		invoke	SvcTerminate, eax
	.else
		; Registration successful - set up SERVICE_STATUS members
		mov		SvcStatus.dwServiceType, cSvcStartType
		mov		SvcStatus.dwServiceSpecificExitCode, 0
		mov		SvcStatus.dwControlsAccepted, cSvcControls
		
		; Report initial status to SCM
		invoke ReportSvcStatus, SERVICE_START_PENDING, NO_ERROR, 1, 5000
	
		; Create the Stop event (set by ScvCtrlHandler when told to stop)
		invoke	CreateEvent, NULL, TRUE, FALSE, NULL
		mov		hSvcStopEvent, eax
		
		; Quit if we couldn't get the Event
		.if eax == NULL
			invoke	GetLastError
			invoke	SvcLogError, ADDR szCreateEvent, eax, TRUE 

			; Terminate the service
			invoke	SvcTerminate, eax
		.else
			; Event created update the Status	
			invoke ReportSvcStatus, SERVICE_START_PENDING, NO_ERROR, 2, 5000
			
			; Perform initialization and start service thread
			invoke	SvcInit, dwArgc, lpszArgv
			invoke	ResumeThread, hThread

			; The service thread is now running
			mov		bServicePaused, FALSE
			
			.if eax == NULL
				; Failed - log error and quit
				Invoke	GetLastError
				invoke	SvcLogError, ADDR szCreateThread, eax, TRUE
				invoke	SvcTerminate, eax
			.else
				; Everything worked! Notify SCM of great news
				invoke ReportSvcStatus, SERVICE_RUNNING, NO_ERROR, 0, 0
				
				; Wait for someone to stop the service or turn off the machine
				invoke	WaitForSingleObject, hSvcStopEvent, INFINITE
				
				; Here we are - someone did something
				invoke	SvcTerminate, 0
			.endif
		.endif
	.endif
		
	Ret
	
SvcMain endp  

;_______________________________________________________________________________
SvcCtrlHandler proc	dwCtrl: DWORD
	; Handle events received from the SCM (Service Control Manager)
	
	; What shall we do today
	.if dwCtrl == SERVICE_CONTROL_STOP
		; Stop the service
		invoke	ReportSvcStatus, SERVICE_STOP_PENDING, NO_ERROR, 0, 0
		invoke	SetEvent, hSvcStopEvent

	.elseif (dwCtrl == SERVICE_CONTROL_PAUSE) && (bServicePaused == FALSE)
		; Pause the service
		invoke	ReportSvcStatus, SERVICE_PAUSE_PENDING, NO_ERROR, 1, 5000
		invoke	SuspendThread, hThread
		mov		bServicePaused, TRUE
		invoke	ReportSvcStatus, SERVICE_PAUSED, NO_ERROR, 0, 0

	.elseif (dwCtrl == SERVICE_CONTROL_CONTINUE) && (bServicePaused == TRUE)
		; Resume the service
		invoke	ReportSvcStatus, SERVICE_CONTINUE_PENDING, NO_ERROR, 1, 5000
		invoke	ResumeThread, hThread
		mov		bServicePaused, FALSE
		invoke	ReportSvcStatus, SERVICE_RUNNING, NO_ERROR, 0, 0

	.else
		; Report the status for any other Ctrl code
		invoke	ReportSvcStatus, SvcStatus.dwCurrentState, NO_ERROR, 0, 0
	.endif

	Ret
	
SvcCtrlHandler endp

;_______________________________________________________________________________
ReportSvcStatus proc dwCurrentState: DWORD, dwWin32ExitCode: DWORD, dwCheckPoint: DWORD, dwWaitHint: DWORD
	; Keep the SCM informed of our every move
	
	; Fill in the Status structure
	mov		eax, dwCurrentState
	mov		SvcStatus.dwCurrentState, eax 
	mov		eax, dwWin32ExitCode
	mov		SvcStatus.dwWin32ExitCode, eax
	mov		eax, dwCheckPoint
	mov		SvcStatus.dwCheckPoint,	eax
	mov		eax, dwWaitHint
	mov		SvcStatus.dwWaitHint, eax
	
	; Chat with the SCM
	.if		dwCurrentState == SERVICE_START_PENDING
		; We are starting so we can't do anything else right now
		mov	SvcStatus.dwControlsAccepted, 0
	.else
		; The SCM can ask us to do whatever we have defined in cSvcControls 
		mov	SvcStatus.dwControlsAccepted, cSvcControls
	.endif
	
	; Let SCM know what we can currently process
	invoke	SetServiceStatus, hSvcStatus, ADDR SvcStatus
	
	Ret
	
ReportSvcStatus endp

;_______________________________________________________________________________
SvcInit proc dwArgc: DWORD, lpszArgv: DWORD
	; Perform the Service initialization
	
	; Create the service thread but don't start it yet
	Invoke	CreateThread, NULL, 0, SvcThread, 0, CREATE_SUSPENDED, NULL
	
	; Save the thread handle
	mov		hThread, eax

	; The service thread is now paused
	mov		bServicePaused, TRUE
	
	; TODO - Add any other initialization code here

	Ret
	
SvcInit endp

;_______________________________________________________________________________
SvcThread proc param: DWORD
	; This proc does all the real work ;)
	
	; This loop goes on for ever. The thread ends by a call to TerminateThread
NeverEndingLoop:

	; *TODO - Add your service code here
	; Use MsgWaitForMultipleObjectsEx rather than Sleep if you have a message queue in here

	invoke	Beep, 440, 10
	invoke	Sleep, 1000
		
	jmp		NeverEndingLoop
	
	Ret
	
SvcThread endp

;_______________________________________________________________________________
SvcTerminate proc dError: DWORD
	; Clean up after an error from SvcMain
	
	.if hSvcStopEvent != NULL
		invoke	CloseHandle, hSvcStopEvent
	.endif
	
	; Inform SCM that we are stopping
	.if hSvcStatus != NULL
		invoke	ReportSvcStatus, SERVICE_STOPPED, dError, 0, 0
	.endif
	
	; Kill the thread
	.if hThread != NULL
		invoke	TerminateThread, hThread, 0
		invoke	CloseHandle, hThread
	.endif
	
	Ret
SvcTerminate endp

;_______________________________________________________________________________
SvcLogError proc	pAPIName: DWORD, dError: DWORD, EventLog: DWORD
	; Display or log a message
	; pAPIName		points to the name of the API that was called
	; dError		result of GetLastError or other number to store in message
	; EventLog		switch for text logging
	;	- False: Text to StdOut
	;	- True:  Text to EventLog

	; Translate the error number into text.
	invoke	FormatMessage, FORMAT_MESSAGE_FROM_SYSTEM, NULL, dError, 0, ADDR szTemp, SIZEOF szTemp, NULL
	
	; If translation failed use the default error message
	.if eax == 0
		invoke	wsprintf, ADDR szTemp, ADDR szEventLog, pAPIName, dError
	.endif

	.if		EventLog == FALSE
		invoke	StdOut, pszTemp
	.else
		invoke PutInEventLog, pszTemp
	.endif
	
	Ret
SvcLogError EndP

;_______________________________________________________________________________
PutInEventLog proc	pStr: DWORD
	; Log the Text in pStr into the Event Log
	LOCAL	hEvt:		HANDLE
	
	; Open the event log
	invoke	RegisterEventSource, NULL, ADDR szServiceName
	mov		hEvt, eax		
	
	; Log the message only if RegisterEventSource is ok  
	.if	eax != 0
		invoke	StrLen, pStr
		invoke	ReportEvent, hEvt, EVENTLOG_ERROR_TYPE, 0, 0, NULL, 0, eax , 0, pStr
		invoke  DeregisterEventSource, hEvt
	.endif
	
	Ret
PutInEventLog EndP 

end start